"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const cp = require("child_process");
const fs = require("fs");
const path = require("path");
const ERDUtil_1 = require("../../../helper/ERDUtil");
const json2csv_1 = require("../../../helper/json2csv");
const sfdcLogin_1 = require("../../../helper/sfdcLogin");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages('sfdx-mohanc-plugins', 'md.describe');
class Query extends command_1.SfdxCommand {
    async run() {
        const loginfilename = this.flags.loginfilename || 'NONE';
        const orgname = this.flags.orgname || 'NONE';
        // get the connection
        const conn = this.org.getConnection();
        const erdOutputfileName = this.flags.erdfilename || 'NONE';
        let erdOutputfileNameExt = 'dot';
        if (erdOutputfileName !== 'NONE') {
            erdOutputfileNameExt = path.extname(erdOutputfileName);
        }
        let sobjectnames = this.flags.sobjectnames || 'Account,Contact';
        // file is given for the sobjectList
        const sobjectListFileName = this.flags.sobjectlistfilename || 'NONE';
        if (sobjectListFileName !== 'NONE') {
            sobjectnames = fs.readFileSync(sobjectListFileName, 'utf8');
        }
        const describeFields = ['sobjectName',
            'sobjectLabel',
            'name', 'label',
            'type', 'length',
            'nillable',
            'referenceTo',
            'relationshipName',
            'unique',
            'updateable',
            'inlineHelpText',
            'picklistValues',
            'externalId'
        ];
        function processResult(result, sobject) {
            const erdFields = [];
            const sobjectName = result.name;
            const sobjectLabel = result.label;
            for (const field of result.fields) {
                erdFields.push({
                    name: field.name,
                    type: field.type,
                    length: field.length,
                    referenceTo: field.referenceTo,
                    relationshipName: field.relationshipName
                });
                const picklistReqd = [];
                const picklist = field.picklistValues;
                for (const plItem of picklist) {
                    picklistReqd.push(plItem.label);
                }
                field.picklistValues = picklistReqd;
                // TODO: Check
                field['sobjectName'] = sobjectName;
                field['sobjectLabel'] = sobjectLabel;
            }
            const sObjectItem = {};
            sObjectItem['name'] = sobject;
            sObjectItem['fields'] = erdFields;
            sObjects.push(sObjectItem);
            const json2csv = new json2csv_1.JSON2CSVParserUtil(result.fields, { fields: describeFields });
            console.log(json2csv.parse2csv());
            if (objCount === sobjects.length) {
                const erdu = new ERDUtil_1.ERDUtil();
                const erdDotStr = erdu.getDotContent(sObjects, 'short');
                if (erdOutputfileName !== 'NONE' && erdOutputfileNameExt === '.dot') {
                    // console.log('dot path');
                    fs.writeFile(erdOutputfileName, erdDotStr, err => {
                        if (err) {
                            console.log(err);
                        }
                    });
                }
                else if (erdOutputfileName !== 'NONE' && erdOutputfileNameExt === '.svg') {
                    fs.writeFile(erdOutputfileName + '.dot', erdDotStr, err => {
                        if (err) {
                            console.log(err);
                        }
                        else {
                            // look for the command dot in the OS and run the conversion
                            if (erdOutputfileNameExt === '.svg') {
                                // console.log('trying to run dot to create svg file...');
                                cp.exec(`dot -Tsvg -o${erdOutputfileName} ${erdOutputfileName}.dot`);
                            }
                        }
                    });
                }
                else {
                    // console.info(`ERD file not defined: ${erdOutputfileName} extn: ${erdOutputfileNameExt}`);
                }
                // console.log(erdDotStr);
            }
        }
        if (sobjectnames === 'FILE') {
            sobjectnames = fs.readFileSync('ObjectList.txt', 'utf8').replace(/\n/g, ',');
            sobjectnames = sobjectnames.replace(/,$/, ''); // remove terminal comma
            console.log(sobjectnames);
        }
        const sobjects = sobjectnames.split(',');
        // console.log(sobjects);
        const sObjects = [];
        let objCount = 0;
        for (const sobject of sobjects) {
            objCount++;
            if (loginfilename !== 'NONE') { // sandbox/prod access
                const forceLogin = new sfdcLogin_1.SfdcLogin(orgname);
                const sfdcConn = await forceLogin.sfdcLogin(loginfilename);
                // console.log(`*** userInfo: ${JSON.stringify(sfdcConn.userInfo, null, 2)}`);
                const result = await sfdcConn.conn.describe(sobject.trim());
                processResult(result, sobject);
            }
            else { // scratch org
                const result = await conn.describe(sobject.trim());
                processResult(result, sobject);
            }
        }
        // Return an object to be displayed with --json
        return {};
    }
}
Query.description = messages.getMessage('commandDescription');
Query.examples = [
    `
   ## for scratch-org:
   sfdx mohanc:md:describe -s RecordType
   # For Sandbox/DE/PROD: Establish connection with:
   #   In case of sandbox  use :    sfdx force:auth:web:login -r https://test.salesforce.com
   #   In case of  PROD or DE use : sfdx force:auth:web:login -r https://login.salesforce.com
   
   ## for DE or sandbox or PROD, when you do not want to use OAUTH2:
   # Sample login.json :https://mohan-chinnappan-n.github.io/dx/samples/login-sample.json
   # sfdx mohanc:md:describe -s RecordType -l login.json -o org40
   
   ## ---- For full org metadata export: scratch org ----
   sfdx mohanc:md:describeGlobal   > ObjectList.txt
   sfdx mohanc:md:describe  -i ObjectList.txt
   ## For full org metadata export: sandbox/DE/PROD org
   sfdx mohanc:md:describeGlobal -u username   > ObjectList.txt
   sfdx mohanc:md:describe -u username -i ObjectList.txt > org-md.csv

   #-----------------------------------------------------------------------
   ## ---- how to draw ER Diagram for the given SObjects along with model export for them ----
   ## SObjects list is comma-separated as shown below:
   ### If you have dot program installed in your OS, ERD will be drawn in svg format :
   sfdx mohanc:md:describe -s Contact,Account -e contact-account.svg > contact-account.csv

   ### for Sandbox/DE/PROD
   sfdx mohanc:md:describe  -u USERNAME -s Contact,Account -e contact-account.svg > contact-account.csv

   ### If you do NOT have dot program installed in your OS, ERD will be drawn in dot format :
   sfdx mohanc:md:describe -s Contact,Account -e contact-account.dot > contact-account.csv
   ### You can use our online dot-viewer: https://mohan-chinnappan-n.github.io/viz/viz.html  to view the model


    `
];
Query.flagsConfig = {
    sobjectnames: command_1.flags.string({ char: 's', description: messages.getMessage('sobjectNames') }),
    loginfilename: command_1.flags.string({ char: 'l', description: messages.getMessage('loginfileNameDescription') }),
    orgname: command_1.flags.string({ char: 'o', description: messages.getMessage('orgnameDescription') }),
    erdfilename: command_1.flags.string({ char: 'e', description: messages.getMessage('erdfilenameDescription') }),
    sobjectlistfilename: command_1.flags.string({ char: 'i', description: messages.getMessage('sobjectlistfilenameDescription') })
};
// Comment this out if your command does not require an org username
Query.requiresUsername = true;
// Comment this out if your command does not support a hub org username
Query.supportsDevhubUsername = true;
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
Query.requiresProject = false;
exports.default = Query;
//# sourceMappingURL=describe.js.map